/*
** Copyright (C) 2010 Yves LE PROVOST
** $Id$
** 
** This program is free software; you can redistribute it and/or modify
** it under the terms of the GNU General Public License as published by
** the Free Software Foundation; either version 2 of the License, or
** (at your option) any later version.
**
** This program is distributed in the hope that it will be useful,
** but WITHOUT ANY WARRANTY; without even the implied warranty of
** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
** GNU General Public License for more details.
**
** You should have received a copy of the GNU General Public License
** along with This program; if not, write to the Free Software
** Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
** 
** Additional permission under GNU GPL version 3 section 7
**
** If you modify this program, or any covered work, by linking or
** combining it with the OpenSSL project's OpenSSL library (or a
** modified version of that library), containing parts covered by the
** terms of the OpenSSL or SSLeay licenses, the Free Software Foundation
** grants you additional permission to convey the resulting work.
** Corresponding Source for a non-source form of such a combination
** shall include the source code for the parts of OpenSSL used as well
** as that of the covered work.  
*/

#include <string.h>
#include <stdio.h>

#include "send.h"
#include "error.h"
#include "http.h"
#include "base64.h"

unsigned int get_contentlength(char *buf, int size)
{
	char * tmp=NULL;
	unsigned int i=15,j=0;
	char charlength[16];

	tmp = strcasestr(buf, "Content-Length: ");
	if(tmp == NULL)
		return 0;
	
	while(tmp[i] == ' ' && i < size)
		i++;

	while(((int) tmp[i])> 47 && ((int) tmp[i])< 58 && j<15 && i < size  )
	{
		charlength[j++]=tmp[i]; 	
		i++;	
	}

	charlength[j]='\0';

	if(DEBUG)
		printf("Content-Length : %i\n", atoi(charlength));

	return (unsigned int) atoi(charlength);
}

bool get_connectionclose(char *buf, int size)
{
	char * tmp=NULL;
	unsigned int i=11;
	
	tmp = strcasestr(buf, "Connection:");
	if(tmp == NULL)
		return false;

	while(tmp[i] == ' ' && i < size)
		i++;

	if(strncasecmp(tmp+i, "close", 5)==0)
		return true;

	return false;

}

bool get_chunked(char *buf, int size)
{

	char *tmp=NULL;
	unsigned int i=18;
	
	if(strlen(buf) < 18)
		return false;	
	tmp = strcasestr(buf, "Transfer-Encoding:");
	if(tmp == NULL)
		return false;

	while(tmp[i] == ' ' && i < size)
		i++;

	if(strncasecmp(tmp+i, "chunked", 7)==0)
		return true;

	return false;

}



bool end_chunck(char *buf, int size)
{
	char *tmp=NULL;
	if(!strncmp(buf, "0\r\n",3))
		return true;	

	tmp = strcasestr(buf, "\r\n0\r\n");
	if(tmp == NULL)
		return false;

	return true;
}

unsigned int get_effective_length(char * buf, int size)
{
	char *tmp;
	unsigned int i;
	tmp = memmem(buf, size, "\r\n\r\n", 4);
	if(tmp==NULL)	
		return(0);

	if(DEBUG)
		printf("Less head : %s\n", tmp);
	i = (unsigned int) ((unsigned long) size - (((unsigned long) tmp) - ((unsigned long) buf) + 4));
	return i;
}

int post(
	char **ressource, 
	t_datathread *data, 
	char *word1, 
	char *word2)
{
	char url[MAX_SIZE_URL];
	int size;
	char length[4];
	char *auth=NULL;

	memset(url, 0, MAX_SIZE_URL);

	strncpy(url, "POST ", 5);
	url[5]='\0';

	if( (strlen(url)+strlen(data->opt.url.url_pre) + 1)>MAX_SIZE_URL)
		return(-2);

	strncat(url,data->opt.url.url_pre, strlen(data->opt.url.url_pre));

	if( (strlen(url) + 17 + strlen(data->opt.url.host) + 2 + 1)> MAX_SIZE_URL)
		return(-2);

	strncat(url," HTTP/1.1\r\nHost: ", 17);
	strncat(url,data->opt.url.host, strlen(data->opt.url.host));
	strncat(url,"\r\n", 2);
	
	if(data->opt.agressive)
	{
		if( (strlen(url) + 22 + 2 + 1)> MAX_SIZE_URL)
		{
			strncat(url,"Connection: Keep-alive",22); 
			strncat(url,"\r\n", 2);
		}
	}

	if(data->opt.headers != NULL)
	{
		if( (strlen(url) + strlen(data->opt.headers) + 1 ) > MAX_SIZE_URL)
			return(-2);
		strncat(url, data->opt.headers, strlen(data->opt.headers));
	}

	size=strlen(data->opt.post.post_pre);
	if(data->opt.post.post_mid != NULL)
		size+=strlen(data->opt.post.post_mid);
	if(data->opt.post.post_end != NULL)
		size+=strlen(data->opt.post.post_end);
	if(word1!=NULL)
		size+=strlen(word1);
	if(word2!=NULL && data->opt.twofuzz)
		size+=strlen(word2);

	if(size<1000)
		sprintf(length, "%i", size);
	else
		exit(0);

	if( (strlen(url) + 16 + strlen(length) + 1 ) > MAX_SIZE_URL)
		return(-2);

	strncat(url, "Content-length: ",16);
	strncat(url, length, strlen(length));

	if ( (strlen(url) +3) > MAX_SIZE_URL)
		return(-2);

	strncat(url,"\r\n", 2);

	if(data->opt.auth.user!=NULL && data->opt.auth.pass!=NULL)
	{
		auth=basic_authent(data->opt.auth.user, data->opt.auth.pass);
		if( (strlen(url) +21+ strlen(auth) +2+1 ) > MAX_SIZE_URL)
			return(-2);
		strcat(url, "Authorization: Basic ");
		strcat(url, auth);
		strcat(url, "\r\n");
		FREE_BUF(auth);
	}

	if ( (strlen(url) +3) > MAX_SIZE_URL)
		return(-2);

	strncat(url,"\r\n", 2);

	if( (strlen(url)+strlen(data->opt.post.post_pre))>MAX_SIZE_URL)
		return(-2);

	strncat(url,data->opt.post.post_pre, strlen(data->opt.post.post_pre));

	if((strlen(url)+strlen(word1)+1)>MAX_SIZE_URL)
		return(-2);

	strncat(url,word1, strlen(word1));

	if(data->opt.post.post_mid!=NULL)
	{
		if( (strlen(url)+strlen(data->opt.post.post_mid)+1 > MAX_SIZE_URL))
			return (-2);
		strncat(url, data->opt.post.post_mid, strlen(data->opt.post.post_mid));
	}

	if(data->opt.twofuzz)
	{
		if( (strlen(url)+strlen(word2)+1>MAX_SIZE_URL))
			return(-2);

		strncat(url,word2, strlen(word2));
	}

	if(data->opt.post.post_end!=NULL)
	{
		if( (strlen(url)+strlen(data->opt.post.post_end)+1)>MAX_SIZE_URL)
			return (-2);
	
		strncat(url, data->opt.post.post_end, strlen(data->opt.post.post_end));
	}

	*ressource = calloc(strlen(url),1);
	EXIT_IFNULL(*ressource, "Memory Error");

	strncpy(*ressource, url, strlen(url));

	return(1);
}

int get(
	char **ressource, 
	t_datathread *data, 
	char *word1, 
	char *word2)
{
	char url[MAX_SIZE_URL];
	memset(url, 0, MAX_SIZE_URL);
	char *auth=NULL;

	strncpy(url, "GET ", 4);
	url[4]='\0';

	if( (strlen(url)+strlen(data->opt.url.url_pre))>MAX_SIZE_URL)
		return(-2);

	strncat(url,data->opt.url.url_pre, strlen(data->opt.url.url_pre));
	
	if(data->opt.urlfuzz)
	{
		if(word1 == NULL || ((strlen(url)+strlen(word1)+1)>MAX_SIZE_URL))
			return(-2);
		strncat(url,word1, strlen(word1));
	}

	if(data->opt.url.url_mid!=NULL)
	{
		if( (strlen(url)+strlen(data->opt.url.url_mid)+1 > MAX_SIZE_URL))
			return (-2);
		strncat(url, data->opt.url.url_mid, strlen(data->opt.url.url_mid));
	}

	if(data->opt.twofuzz)
	{
		if( (strlen(url)+strlen(word2)+1>MAX_SIZE_URL))
			return(-2);

		strncat(url,word2, strlen(word2));
	}

	if(data->opt.url.url_end!=NULL)
	{
		if( (strlen(url)+strlen(data->opt.url.url_end)+1)>MAX_SIZE_URL)
			return (-2);
	
		strncat(url, data->opt.url.url_end, strlen(data->opt.url.url_end));
	}

	if( (strlen(url) + 17 + strlen(data->opt.url.host) + 2 + 1)> MAX_SIZE_URL)
		return(-2);

	strncat(url," HTTP/1.1\r\nHost: ", 17);
	strncat(url,data->opt.url.host, strlen(data->opt.url.host));
	strncat(url,"\r\n", 2);

	if(data->opt.agressive)
	{
		if( (strlen(url) + 22 + 2 + 1)> MAX_SIZE_URL)
			return(-2);
		strncat(url,"Connection: Keep-alive",22); 
		strncat(url,"\r\n", 2);
	}

	if(data->opt.headers != NULL)
	{
		if( (strlen(url) + strlen(data->opt.headers) + 1 ) > MAX_SIZE_URL)
			return(-2);
		strncat(url, data->opt.headers, strlen(data->opt.headers));
	}

	if(data->opt.auth.user!=NULL && data->opt.auth.pass!=NULL)
	{
		auth=basic_authent(data->opt.auth.user, data->opt.auth.pass);
		if( (strlen(url) +21+ strlen(auth) +2+1 ) > MAX_SIZE_URL)
			return(-2);
		strcat(url, "Authorization: Basic ");
		strcat(url, auth);
		strcat(url, "\r\n");
		FREE_BUF(auth);	
	}

	if ( (strlen(url) +3) > MAX_SIZE_URL)
		return(-2);

	strncat(url,"\r\n", 2);

	*ressource = calloc(strlen(url)+1,1);
	memset(*ressource, 0, sizeof(*ressource));
	EXIT_IFNULL(*ressource, "Memory Error");

	strncpy(*ressource, url, strlen(url));

	return(1);
}

char *basic_authent(char *user, char *pass)
{
	char *plain=NULL;
	char *encod=NULL;
	int input_size=0;

	plain=calloc(strlen(user)+strlen(pass)+2,1);
	strcpy(plain, user);
	strcat(plain, ":");
	strcat(plain, pass);
	//encod=calloc(strlen(plain)+strlen(plain +1) + 1,1);
	input_size=strlen(plain);
	encod=calloc(((input_size * 4) / 3) + ((input_size % 3) ? (4 - (input_size % 3)) : 0) +1, 1);
	base64encod(plain, encod, input_size);
	return encod;
}

int basic_auth(
	char **ressource, 
	t_datathread *data, 
	char *word1, 
	char *word2)
{
	char url[MAX_SIZE_URL];
	memset(url, 0, MAX_SIZE_URL);
	char *auth=NULL;

	strncpy(url, "GET ", 4);
	url[4]='\0';

	if( (strlen(url)+strlen(data->opt.url.url_pre))>MAX_SIZE_URL)
		return(-2);

	strncat(url,data->opt.url.url_pre, strlen(data->opt.url.url_pre));
	
	if( (strlen(url) + 17 + strlen(data->opt.url.host) + 2 + 1)> MAX_SIZE_URL)
		return(-2);

	strncat(url," HTTP/1.1\r\nHost: ", 17);
	strncat(url,data->opt.url.host, strlen(data->opt.url.host));
	strncat(url,"\r\n", 2);

	if(data->opt.agressive)
	{
		if( (strlen(url) + 22 + 2 + 1)> MAX_SIZE_URL)
			return(-2);
		strncat(url,"Connection: Keep-alive",22); 
		strncat(url,"\r\n", 2);
	}

	if(data->opt.headers != NULL)
	{
		if( (strlen(url) + strlen(data->opt.headers) + 1 ) > MAX_SIZE_URL)
			return(-2);
		strncat(url, data->opt.headers, strlen(data->opt.headers));
	}

	if(data->opt.auth.user==NULL && data->opt.auth.pass==NULL)
		auth=basic_authent(word1, word2);
	if(data->opt.auth.user!=NULL && data->opt.auth.pass==NULL)
		auth=basic_authent(data->opt.auth.user, word1);
	if(data->opt.auth.user==NULL && data->opt.auth.pass!=NULL)
		auth=basic_authent(word1, data->opt.auth.pass);
	
	if( (strlen(url) +21+ strlen(auth) +2+1 ) > MAX_SIZE_URL)
			return(-2);
		strcat(url, "Authorization: Basic ");
		strcat(url, auth);
		strcat(url, "\r\n");
		FREE_BUF(auth);	

	if ( (strlen(url) +3) > MAX_SIZE_URL)
		return(-2);

	strncat(url,"\r\n", 2);

	*ressource = calloc(strlen(url)+1,1);
	memset(*ressource, 0, sizeof(*ressource));
	EXIT_IFNULL(*ressource, "Memory Error");

	strncpy(*ressource, url, strlen(url));

	return(1);
}

